@@ -3,18 +3,86 @@
 import timeit
 import unittest
 
-from wpull.url import (URLInfo, schemes_similar, is_subdir, unquote,
-                       unquote_plus, quote, quote_plus, split_query,
-                       uppercase_percent_encoding, urljoin,
-                       flatten_path, is_likely_link, is_unlikely_link)
+from wpull.url import URLInfo, schemes_similar, is_subdir, split_query, \
+    percent_decode, percent_decode_plus, percent_encode, percent_encode_plus, \
+    uppercase_percent_encoding, urljoin, flatten_path, parse_url_or_log
 
 
 class TestURL(unittest.TestCase):
-    def test_url_info(self):
+    @unittest.skip('experiment only')
+    def test_lib_vs_wpull(self):
+        result_1 = timeit.timeit('''
+        from urllib.parse import urlsplit
+        for i in range(1000):
+            urlsplit('http://donkey{i}.com/waffles{i}'.format(i=i))
+        ''', number=100)
+        result_2 = timeit.timeit('''
+        from wpull.url import URLInfo
+        parse = URLInfo.parse
+        for i in range(1000):
+            parse('http://donkey{i}.com/waffles{i}'.format(i=i))
+        ''', number=100)
+
+        print(result_1, result_2)
+
+    def test_url_info_naked(self):
         self.assertEqual(
             'http://example.com/',
             URLInfo.parse('example.com').url
         )
+        self.assertEqual(
+            'http://example.com/',
+            URLInfo.parse('//example.com').url
+        )
+        self.assertEqual(
+            'http://example.com/blah',
+            URLInfo.parse('//example.com/blah').url
+        )
+
+        url_info = URLInfo.parse('example.com:8080')
+        self.assertEqual('http://example.com:8080/', url_info.url)
+        self.assertEqual('example.com:8080', url_info.hostname_with_port)
+        self.assertEqual(8080, URLInfo.parse('example.com:8080').port)
+
+        self.assertEqual(
+            'http://example.com/asdf',
+            URLInfo.parse('example.com/asdf#blah').url
+        )
+        self.assertEqual(
+            'http://example.com/asdf/ghjk',
+            URLInfo.parse('example.com/asdf/ghjk#blah').url
+        )
+        self.assertEqual(
+            'http://example.com/',
+            URLInfo.parse('example.com/').url
+        )
+        self.assertEqual(
+            'https://example.com/',
+            URLInfo.parse('https://example.com').url
+        )
+
+    def test_url_info_parts(self):
+        url_info = URLInfo.parse(
+            'HTTP://userName:pass%3Aword@[::1]:81/ásdf/ghjk?a=b=c&d#/?')
+        self.assertEqual('http://[::1]:81/%C3%A1sdf/ghjk?a=b=c&d', url_info.url)
+        self.assertEqual('http', url_info.scheme)
+        self.assertEqual('userName:pass%3Aword@[::1]:81',
+                         url_info.authority)
+        self.assertEqual('/ásdf/ghjk?a=b=c&d#/?', url_info.resource)
+        self.assertEqual('userName', url_info.username)
+        self.assertEqual('pass:word', url_info.password)
+        self.assertEqual('[::1]:81', url_info.host)
+        self.assertEqual('::1', url_info.hostname)
+        self.assertEqual(81, url_info.port)
+        self.assertEqual('/%C3%A1sdf/ghjk', url_info.path)
+        self.assertEqual('a=b=c&d', url_info.query)
+        self.assertEqual('/?', url_info.fragment)
+        self.assertEqual('utf-8', url_info.encoding)
+        self.assertEqual(
+            'HTTP://userName:pass%3Aword@[::1]:81/ásdf/ghjk?a=b=c&d#/?',
+            url_info.raw)
+
+    def test_url_info_default_port(self):
         self.assertEqual(
             80,
             URLInfo.parse('http://example.com').port
@@ -31,104 +99,15 @@ class TestURL(unittest.TestCase):
             'example.com',
             URLInfo.parse('https://example.com').hostname_with_port
         )
-        self.assertEqual(
-            'http://example.com/',
-            URLInfo.parse('example.com/').url
-        )
-        self.assertEqual(
-            'https://example.com/',
-            URLInfo.parse('https://example.com').url
-        )
-        self.assertEqual(
-            'http://example.com/',
-            URLInfo.parse('//example.com').url
-        )
-        self.assertEqual(
-            'http://example.com:8080/',
-            URLInfo.parse('example.com:8080').url
-        )
-        self.assertEqual(
-            'example.com:8080',
-            URLInfo.parse('example.com:8080').hostname_with_port
-        )
-        self.assertEqual(
-            8080,
-            URLInfo.parse('example.com:8080').port
-        )
-        self.assertEqual(
-            'http://example.com/asdf',
-            URLInfo.parse('example.com/asdf#blah').url
-        )
-        self.assertEqual(
-            'http://example.com/asdf/ghjk',
-            URLInfo.parse('example.com/asdf/ghjk#blah').url
-        )
-
-        url_info = URLInfo.parse(
-            'HTTP://username:password@example.com/asdf/ghjk/')
-        self.assertEqual(
-            'http://example.com/asdf/ghjk/',
-            url_info.url
-        )
-        self.assertEqual(
-            'http',
-            url_info.scheme
-        )
-        self.assertEqual(
-            'username',
-            url_info.username
-        )
-        self.assertEqual(
-            'password',
-            url_info.password
-        )
-        self.assertEqual(
-            'utf-8',
-            url_info.encoding
-        )
-
-        self.assertEqual(
-            'http://example.com/%C3%B0',
-            URLInfo.parse('http://example.com/ð').url
-        )
-
-        url_info = URLInfo.parse('mailto:user@example.com')
-        self.assertEqual(
-            'mailto:user@example.com',
-            url_info.url
-        )
-        self.assertEqual(
-            'mailto',
-            url_info.scheme
-        )
-
         self.assertEqual(
             'http://example.com/',
             URLInfo.parse('http://example.com:80').url
         )
 
-        # URL parsing is really different in each version of Python...
-        self.assertRaises(ValueError, URLInfo.parse, '')
-        self.assertRaises(ValueError, URLInfo.parse, '#')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://')
-        self.assertRaises(ValueError, URLInfo.parse, 'example....com')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://example....com')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://example…com')
-#         self.assertRaises(ValueError, URLInfo.parse, 'http://[34.4kf]::4')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://[34.4kf::4')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://dmn3]:3a:45')
-        self.assertRaises(ValueError, URLInfo.parse, ':38/3')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://][a:@1]')
-#         self.assertRaises(ValueError, URLInfo.parse, 'http://[[aa]]:4:]6')
-        self.assertNotIn('[', URLInfo.parse('http://[a]').hostname)
-        self.assertNotIn(']', URLInfo.parse('http://[a]').hostname)
-        self.assertRaises(ValueError, URLInfo.parse, 'http://[[a]')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://[[a]]a]')
-        self.assertRaises(ValueError, URLInfo.parse, 'http://[[a:a]]')
-
+    def test_url_info_percent_encode(self):
         self.assertEqual(
-            'http://example.com/blah',
-            URLInfo.parse('//example.com/blah').url
+            'http://example.com/%C3%B0',
+            URLInfo.parse('http://example.com/ð').url
         )
         self.assertEqual(
             'http://example.com/blah%20blah/',
@@ -195,25 +174,65 @@ class TestURL(unittest.TestCase):
 
         self.assertEqual(
             'http://example.com/'
-            '?blah=http://example.com/?fail=true',
+            '?blah=http%3A%2F%2Fexample.com%2F%3Ffail%3Dtrue',
             URLInfo.parse(
                 'http://example.com/'
                 '?blah=http%3A%2F%2Fexample.com%2F%3Ffail%3Dtrue').url
         )
         self.assertEqual(
             'http://example.com/'
-            '?blah=http://example.com/?fail=true',
+            '?blah=http://example.com/?fail%3Dtrue',
             URLInfo.parse(
                 'http://example.com/'
                 '?blah=http://example.com/?fail%3Dtrue').url
         )
 
         self.assertEqual(
-            'http://example.com/??blah=blah[0:]=blah?blah%22&d%26_',
+            'http://example.com/??blah=blah[0:]=bl%61h?blah%22&d%26_',
             URLInfo.parse(
                 'http://example.com/??blah=blah[0:]=bl%61h?blah"&d%26_').url
         )
 
+    def test_url_info_not_http(self):
+        url_info = URLInfo.parse('mailto:user@example.com')
+        self.assertEqual('mailto:user@example.com', url_info.url)
+        self.assertEqual('mailto', url_info.scheme)
+
+    def test_url_info_invalids(self):
+        self.assertRaises(ValueError, URLInfo.parse, '')
+        self.assertRaises(ValueError, URLInfo.parse, '#')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://')
+        self.assertRaises(ValueError, URLInfo.parse, 'example....com')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://example....com')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://example…com')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://[34.4kf]::4')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://[34.4kf::4')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://dmn3]:3a:45')
+        self.assertRaises(ValueError, URLInfo.parse, ':38/3')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://][a:@1]')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://[[aa]]:4:]6')
+        self.assertNotIn('[', URLInfo.parse('http://[a]').hostname)
+        self.assertNotIn(']', URLInfo.parse('http://[a]').hostname)
+        self.assertRaises(ValueError, URLInfo.parse, 'http://[[a]')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://[[a]]a]')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://[[a:a]]')
+        self.assertRaises(ValueError, URLInfo.parse, 'http:///')
+        self.assertRaises(ValueError, URLInfo.parse, 'http:///horse')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://?what?')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://#egg=wpull')
+        self.assertRaises(ValueError, URLInfo.parse,
+                          'http://:@example.com:?@/')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://\x00/')
+        self.assertRaises(ValueError, URLInfo.parse, 'http:/a')
+        self.assertRaises(ValueError, URLInfo.parse, 'http://@@example.com/@')
+        self.assertRaises(
+            ValueError, URLInfo.parse,
+            'http://ｆａｔ３２ｄｅｆｒａｇｍｅｎｔｅｒ.internets：：８０')
+        self.assertRaises(
+            ValueError, URLInfo.parse,
+            'http://ｆａｔ３２ｄｅｆｒａｇｍｅｎｔｅｒ.internets：８０/')
+
+    def test_url_info_path_folding(self):
         self.assertEqual(
             'http://example.com/',
             URLInfo.parse('http://example.com/../').url
@@ -230,6 +249,8 @@ class TestURL(unittest.TestCase):
             'http://example.com/a/style.css',
             URLInfo.parse('http://example.com/a/b/../style.css').url
         )
+
+    def test_url_info_reserved_char_is_ok(self):
         self.assertEqual(
             'http://example.com/@49IMG.DLL/$SESSION$/image.png;large',
             URLInfo.parse(
@@ -241,14 +262,41 @@ class TestURL(unittest.TestCase):
                 'http://example.com/@49IMG.DLL/$SESSION$/imagé.png;large').url
         )
         self.assertEqual(
-            'http://example.com/$c/%25system.exe/',
+            'http://example.com/$c/%system.exe/',
             URLInfo.parse('http://example.com/$c/%system.exe/').url
         )
 
+    def test_url_info_misleading_parts(self):
         self.assertEqual(
             'http://example.com/?a',
             URLInfo.parse('http://example.com?a').url
         )
+        self.assertEqual(
+            'http://example.com/?a?',
+            URLInfo.parse('http://example.com?a?').url
+        )
+        self.assertEqual(
+            'http://example.com/',
+            URLInfo.parse('http://example.com#a').url
+        )
+        self.assertEqual(
+            'http://example.com/',
+            URLInfo.parse('http://example.com#a?').url
+        )
+        self.assertEqual(
+            'http://example.com/?a',
+            URLInfo.parse('http://example.com?a#').url
+        )
+        self.assertEqual(
+            'http://example.com/:10',
+            URLInfo.parse('http://example.com/:10').url
+        )
+        self.assertEqual(
+            'http://example.com/?@/',
+            URLInfo.parse('http://:@example.com?@/').url
+        )
+
+    def test_url_info_query(self):
         self.assertEqual(
             'http://example.com/?a=',
             URLInfo.parse('http://example.com?a=').url
@@ -265,6 +313,8 @@ class TestURL(unittest.TestCase):
             'http://example.com/?a=1&b=',
             URLInfo.parse('http://example.com?a=1&b=').url
         )
+
+    def test_url_info_ipv6(self):
         self.assertEqual(
             'https://[2001:db8:85a3:8d3:1319:8a2e:370:7348]:8080/ipv6',
             URLInfo.parse(
@@ -278,6 +328,17 @@ class TestURL(unittest.TestCase):
             ).url
         )
 
+    def test_url_info_trailing_dot(self):
+        self.assertEqual(
+            'http://example.com./',
+            URLInfo.parse('http://example.com./').url
+        )
+
+        self.assertEqual(
+            'http://example.com.:81/',
+            URLInfo.parse('http://example.com.:81/').url
+        )
+
     def test_url_info_round_trip(self):
         urls = [
             'http://example.com/blah%20blah/',
@@ -329,30 +390,6 @@ class TestURL(unittest.TestCase):
             URLInfo.parse('https://[::ffff:c000:0280]').url
         )
 
-    @unittest.skip('experiment only')
-    def test_url_info_timing(self):
-        t1 = timeit.timeit(
-            '''URLInfo.parse(
-            "http://asdjfklðkjir.com:585?" +
-            "$fasjdfklfd=45asdfasdf345hd.s&g4=4d&&" +
-            str(random.randint(0,1000))
-            )''',
-            number=2000,
-            setup='import random; from wpull.url import URLInfo',
-        )
-        t2 = timeit.timeit(
-            '''URLInfo.parse(
-            "http://asdjfklðkjir.com:585?" +
-            "$fasjdfklfd=45asdfasdf345hd.s&g4=4d&&" +
-            str(random.randint(0,1000))
-            , use_cache=False
-            )''',
-            number=2000,
-            setup='import random; from wpull.url import URLInfo',
-        )
-        print(t1, t2)
-        self.assertLess(t1, t2)
-
     def test_url_info_to_dict(self):
         url_info = URLInfo.parse('https://example.com/file.jpg')
         url_info_dict = url_info.to_dict()
@@ -405,15 +442,15 @@ class TestURL(unittest.TestCase):
         self.assertEqual([('a', 'ð'), ('b', '%2F')],
                          split_query('a=ð&b=%2F'))
 
-    def test_url_quote(self):
-        self.assertEqual('a ', unquote('a%20'))
-        self.assertEqual('að', unquote('a%C3%B0'))
-        self.assertEqual('a ', unquote_plus('a+'))
-        self.assertEqual('að', unquote_plus('a%C3%B0'))
-        self.assertEqual('a%20', quote('a '))
-        self.assertEqual('a%C3%B0', quote('að'))
-        self.assertEqual('a+', quote_plus('a '))
-        self.assertEqual('a%C3%B0', quote_plus('að'))
+    def test_url_percent_encode(self):
+        self.assertEqual('a ', percent_decode('a%20'))
+        self.assertEqual('að', percent_decode('a%C3%B0'))
+        self.assertEqual('a ', percent_decode_plus('a+'))
+        self.assertEqual('að', percent_decode_plus('a%C3%B0'))
+        self.assertEqual('a%20', percent_encode('a '))
+        self.assertEqual('a%C3%B0', percent_encode('að'))
+        self.assertEqual('a+', percent_encode_plus('a '))
+        self.assertEqual('a%C3%B0', percent_encode_plus('að'))
 
     def test_uppercase_percent_encoding(self):
         self.assertEqual(
@@ -492,48 +529,6 @@ class TestURL(unittest.TestCase):
             flatten_path('/dog/../dog/./cat/../doc/././../doc/index.html')
         )
 
-    def test_is_likely_link(self):
-        self.assertTrue(is_likely_link('image.png'))
-        self.assertTrue(is_likely_link('video.mp4'))
-        self.assertTrue(is_likely_link('/directory'))
-        self.assertTrue(is_likely_link('directory/'))
-        self.assertTrue(is_likely_link('/directory/'))
-        self.assertTrue(is_likely_link('../directory/'))
-        self.assertTrue(is_likely_link('http://example.com/'))
-        self.assertTrue(is_likely_link('https://example.com/'))
-        self.assertTrue(is_likely_link('ftp://example.com'))
-        self.assertTrue(is_likely_link('directory/index.html'))
-        self.assertFalse(is_likely_link('directory/another_directory'))
-        self.assertTrue(is_likely_link('application/windows.exe'))
-        self.assertTrue(is_likely_link('//example.com/admin'))
-        self.assertFalse(is_likely_link('12.0'))
-        self.assertFalse(is_likely_link('7'))
-        self.assertFalse(is_likely_link('horse'))
-        self.assertFalse(is_likely_link(''))
-        self.assertFalse(is_likely_link('setTimeout(myTimer, 1000)'))
-        self.assertFalse(is_likely_link('comment.delete'))
-        self.assertFalse(is_likely_link('example.com'))
-        self.assertFalse(is_likely_link('example.net'))
-        self.assertFalse(is_likely_link('example.org'))
-        self.assertFalse(is_likely_link('example.edu'))
-
-    def test_is_unlikely_link(self):
-        self.assertTrue(is_unlikely_link('example.com+'))
-        self.assertTrue(is_unlikely_link('www.'))
-        self.assertTrue(is_unlikely_link(':example.com'))
-        self.assertTrue(is_unlikely_link(',example.com'))
-        self.assertTrue(is_unlikely_link('http:'))
-        self.assertTrue(is_unlikely_link('.example.com'))
-        self.assertTrue(is_unlikely_link('doc[0]'))
-        self.assertTrue(is_unlikely_link('/'))
-        self.assertTrue(is_unlikely_link('//'))
-        self.assertTrue(is_unlikely_link('application/json'))
-        self.assertTrue(is_unlikely_link('application/javascript'))
-        self.assertTrue(is_unlikely_link('text/javascript'))
-        self.assertTrue(is_unlikely_link('text/plain'))
-        self.assertFalse(is_unlikely_link('http://'))
-        self.assertFalse(is_unlikely_link('example'))
-        self.assertFalse(is_unlikely_link('example.com'))
-        self.assertFalse(is_unlikely_link('//example.com/assets/image.css'))
-        self.assertFalse(is_unlikely_link('./image.css'))
-        self.assertFalse(is_unlikely_link('../image.css'))
+    def test_parse_url_or_log(self):
+        self.assertTrue(parse_url_or_log('http://example.com'))
+        self.assertFalse(parse_url_or_log('http://'))
